<?php

namespace EventPortal\Tests\Feature\Admin\Delegate;

use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Delegate\Models\DelegateFormData;
use EventPortal\Domain\Delegate\Models\DelegateFormStep;
use EventPortal\Domain\Event\Form\Enums\State;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Form\Models\FormStepSection;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Form\Models\QuestionValidation;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class RegistrationTest extends TestCase
{
    use RefreshDatabase, SeedRolesAndPermissions;

    /** @test **/
    public function delegate_can_register_for_an_event(): void
    {
        $delegate = factory(Delegate::class)->create();
        $event = factory(Event::class)->state('published')->create();
        $eventDelegate = factory(EventDelegate::class)->state('invited')
            ->create(['delegate_id' => $delegate->id, 'event_id' => $event->id]);
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        $section = factory(FormStepSection::class)->create(['form_step_id' => $step->id]);
        $question = factory(Question::class)
            ->create(['form_step_section_id' => $section->id, 'question' => 'whats your name?']);
        factory(QuestionValidation::class)->state('required')->create(['question_id' => $question->id]);

        $this->actingAsDelegate($delegate)
            ->post(route('delegate.form.store', [
                'event' => $event->slug,
                'form' => $form->slug,
                'step' => $step,
            ]), ['whats_your_name' => 'John Smith']);

        $this->assertEquals('John Smith', DelegateFormData::first()->value);
        $this->assertEquals(State::Complete(), DelegateFormStep::first()->state);
        $this->assertEquals('registered', $eventDelegate->refresh()->status);
    }

    /** @test **/
    public function delegate_can_be_marked_as_registering_for_an_event(): void
    {
        $delegate = factory(Delegate::class)->create();
        $event = factory(Event::class)->state('published')->create();
        $eventDelegate = factory(EventDelegate::class)->state('invited')
            ->create(['delegate_id' => $delegate->id, 'event_id' => $event->id]);
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step1 = factory(FormStep::class)->create(['form_id' => $form->id]);
        $step2 = factory(FormStep::class)->create(['form_id' => $form->id]);
        $section1 = factory(FormStepSection::class)->create(['form_step_id' => $step1->id]);
        $section2 = factory(FormStepSection::class)->create(['form_step_id' => $step2->id]);
        $question1 = factory(Question::class)
            ->create(['form_step_section_id' => $section1->id, 'question' => 'whats your first name?']);
        $question2 = factory(Question::class)
            ->create(['form_step_section_id' => $section2->id, 'question' => 'whats your last name?']);
        factory(QuestionValidation::class)->state('required')->create(['question_id' => $question1->id]);
        factory(QuestionValidation::class)->state('required')->create(['question_id' => $question2->id]);

        $this->actingAsDelegate($delegate)
            ->post(route('delegate.form.store', [
                'event' => $event->slug,
                'form' => $form->slug,
                'step' => $step1,
            ]), ['whats_your_first_name' => 'John']);

        $this->assertEquals('registering', $eventDelegate->refresh()->status);
    }

    /** @test **/
    public function delegate_can_save_progress_for_later_for_an_event(): void
    {
        $delegate = factory(Delegate::class)->create();
        $event = factory(Event::class)->state('published')->create();
        $eventDelegate = factory(EventDelegate::class)->state('invited')
            ->create(['delegate_id' => $delegate->id, 'event_id' => $event->id]);
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step1 = factory(FormStep::class)->create(['form_id' => $form->id]);
        $step2 = factory(FormStep::class)->create(['form_id' => $form->id]);
        $section1 = factory(FormStepSection::class)->create(['form_step_id' => $step1->id]);
        $section2 = factory(FormStepSection::class)->create(['form_step_id' => $step2->id]);
        $question1 = factory(Question::class)
            ->create(['form_step_section_id' => $section1->id, 'question' => 'whats your first name?']);
        $question2 = factory(Question::class)
            ->create(['form_step_section_id' => $section2->id, 'question' => 'whats your last name?']);
        factory(QuestionValidation::class)->state('required')->create(['question_id' => $question1->id]);
        factory(QuestionValidation::class)->state('required')->create(['question_id' => $question2->id]);

        $this->actingAsDelegate($delegate)
            ->post(route('delegate.form.update', [
                'event' => $event->slug,
                'form' => $form->slug,
                'step' => $step1,
            ]), ['whats_your_first_name' => 'John']);

        $this->assertEquals('John', DelegateFormData::first()->value);
        $this->assertEquals(State::Partial(), DelegateFormStep::first()->state);
        $this->assertEquals('registering', $eventDelegate->refresh()->status);
    }

    /** @test **/
    public function delegate_gets_validation_error_when_registering_for_an_event(): void
    {
        $delegate = factory(Delegate::class)->create();
        $event = factory(Event::class)->state('published')->create();
        factory(EventDelegate::class)->state('invited')
            ->create(['delegate_id' => $delegate->id, 'event_id' => $event->id]);
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        $section = factory(FormStepSection::class)->create(['form_step_id' => $step->id]);
        $question = factory(Question::class)
            ->create(['form_step_section_id' => $section->id, 'question' => 'whats your name?']);
        factory(QuestionValidation::class)->state('required')->create(['question_id' => $question->id]);

        $response = $this->actingAsDelegate($delegate)
            ->post(route('delegate.form.store', [
                'event' => $event->slug,
                'form' => $form->slug,
                'step' => $step,
            ]), ['whats_your_name' => '']);

        $response->assertSessionHasErrors(['whats_your_name' => 'The whats your name field is required.']);
    }
}
